@@ -17,7 +17,7 @@ pub struct Tracer<F> {
     max_ttl: TimeToLive,
     grace_duration: Duration,
     max_inflight: MaxInflight,
-    min_sequence: Sequence,
+    initial_sequence: Sequence,
     read_timeout: Duration,
     min_round_duration: Duration,
     max_round_duration: Duration,
@@ -34,7 +34,7 @@ impl<F: Fn(&Probe)> Tracer<F> {
             max_ttl: config.max_ttl,
             grace_duration: config.grace_duration,
             max_inflight: config.max_inflight,
-            min_sequence: config.min_sequence,
+            initial_sequence: config.initial_sequence,
             read_timeout: config.read_timeout,
             min_round_duration: config.min_round_duration,
             max_round_duration: config.max_round_duration,
@@ -46,7 +46,7 @@ impl<F: Fn(&Probe)> Tracer<F> {
     ///
     /// TODO describe algorithm
     pub fn trace<N: Network>(self, mut network: N) -> TraceResult<()> {
-        let mut state = TracerState::new(self.first_ttl, self.min_sequence);
+        let mut state = TracerState::new(self.first_ttl, self.initial_sequence);
         while !state.finished(self.max_rounds) {
             self.send_request(&mut network, &mut state)?;
             self.recv_response(&mut network, &mut state)?;
@@ -219,19 +219,23 @@ mod state {
 
     /// The maximum number of `Probe` entries in the circular buffer.
     ///
-    /// This is effectively also the maximum time-to-live (TTL) we can support.
+    /// This is effectively also the maximum number of time-to-live (TTL) we can support.  We only ever send TTL in
+    /// the range 1..255 so this is technically one larger than we need.
     const BUFFER_SIZE: u16 = 256;
 
     /// The maximum sequence number.
-    const MAX_SEQUENCE: Sequence = Sequence(u16::MAX);
+    ///
+    /// The sequence number is only ever wrapped between rounds and so we need to ensure that there are enough sequence
+    /// numbers for a complete round (i.e. the max TTL, which is `BUFFER_SIZE`).
+    const MAX_SEQUENCE: Sequence = Sequence(u16::MAX - BUFFER_SIZE);
 
     /// Mutable state needed for the tracing algorithm.
     #[derive(Debug)]
     pub struct TracerState {
         /// The state of all `Probe` requests and responses.
         buffer: [Probe; BUFFER_SIZE as usize],
-        /// The minimum sequence number configuration, used to reset sequence when it wraps around.
-        min_sequence: Sequence,
+        /// The initial sequence number configuration, used to reset sequence when it wraps around.
+        initial_sequence: Sequence,
         /// An increasing sequence number for every `EchoRequest`.
         sequence: Sequence,
         /// The starting sequence number of the current round.
@@ -255,12 +259,12 @@ mod state {
     }
 
     impl TracerState {
-        pub fn new(first_ttl: TimeToLive, min_sequence: Sequence) -> Self {
+        pub fn new(first_ttl: TimeToLive, initial_sequence: Sequence) -> Self {
             Self {
                 buffer: [Probe::default(); BUFFER_SIZE as usize],
-                min_sequence,
-                sequence: min_sequence,
-                round_sequence: min_sequence,
+                initial_sequence,
+                sequence: initial_sequence,
+                round_sequence: initial_sequence,
                 ttl: first_ttl,
                 round: Round::from(0),
                 round_start: SystemTime::now(),
@@ -327,15 +331,16 @@ mod state {
         }
 
         /// Create and return the next `Probe` at the current `sequence` and `ttl`.
+        ///
+        /// We post-increment `ttl` here and so in practice we only allow `ttl` values in the range `1..254` to allow
+        /// us to use a `u8`.
         pub fn next_probe(&mut self) -> Probe {
             let probe = Probe::new(self.sequence, self.ttl, self.round, SystemTime::now());
             self.buffer[usize::from(self.sequence % BUFFER_SIZE)] = probe;
+            debug_assert!(self.ttl < TimeToLive(u8::MAX));
             self.ttl += TimeToLive::from(1);
-            if self.sequence == MAX_SEQUENCE {
-                self.sequence = self.min_sequence;
-            } else {
-                self.sequence += Sequence(1);
-            }
+            debug_assert!(self.sequence < Sequence(u16::MAX));
+            self.sequence += Sequence(1);
             probe
         }
 
@@ -378,7 +383,14 @@ mod state {
         }
 
         /// Advance to the next round.
+        ///
+        /// If, during the rond which just completed, we went above the max sequence number then we reset it here.
+        /// We do this here to avoid having to deal with the sequence number wrapping during a round, which is more
+        /// problematic.
         pub fn advance_round(&mut self, first_ttl: TimeToLive) {
+            if self.sequence >= MAX_SEQUENCE {
+                self.sequence = self.initial_sequence;
+            }
             self.target_found = false;
             self.round_sequence = self.sequence;
             self.received_time = None;
@@ -395,6 +407,7 @@ mod state {
         use super::*;
         use crate::tracing::probe::IcmpPacketType;
         use crate::tracing::ProbeStatus;
+        use rand::Rng;
         use std::net::{IpAddr, Ipv4Addr};
 
         #[allow(
@@ -589,6 +602,76 @@ mod state {
                 assert_eq!(probe_3_recv, probe_next2);
             }
         }
+
+        #[test]
+        fn test_sequence_wrap1() {
+            // Start from MAX_SEQUENCE - 1 which is (65279 - 1) == 65278
+            let initial_sequence = Sequence(65278);
+            let mut state = TracerState::new(TimeToLive::from(1), initial_sequence);
+            assert_eq!(state.round, Round(0));
+            assert_eq!(state.sequence, initial_sequence);
+            assert_eq!(state.round_sequence, initial_sequence);
+
+            // Create a probe at seq 65278
+            assert_eq!(state.next_probe().sequence, Sequence(65278));
+            assert_eq!(state.sequence, Sequence(65279));
+
+            // Validate the probes()
+            {
+                let mut iter = state.probes();
+                assert_eq!(iter.next().unwrap().sequence, Sequence(65278));
+                iter.take(BUFFER_SIZE as usize - 1)
+                    .for_each(|p| assert_eq!(p.sequence, Sequence(0)));
+            }
+
+            // Advance the round, which will wrap the sequence back to initial_sequence
+            state.advance_round(TimeToLive::from(1));
+            assert_eq!(state.round, Round(1));
+            assert_eq!(state.sequence, initial_sequence);
+            assert_eq!(state.round_sequence, initial_sequence);
+
+            // Create a probe at seq 65278
+            assert_eq!(state.next_probe().sequence, Sequence(65278));
+            assert_eq!(state.sequence, Sequence(65279));
+
+            // Validate the probes() again
+            {
+                let mut iter = state.probes();
+                assert_eq!(iter.next().unwrap().sequence, Sequence(65278));
+                iter.take(BUFFER_SIZE as usize - 1)
+                    .for_each(|p| assert_eq!(p.sequence, Sequence(0)));
+            }
+        }
+
+        #[test]
+        fn test_sequence_wrap2() {
+            let total_rounds = 2000;
+            let max_probe_per_round = 254;
+            let mut state = TracerState::new(TimeToLive::from(1), Sequence(33000));
+            for _ in 0..total_rounds {
+                for _ in 0..max_probe_per_round {
+                    let _probe = state.next_probe();
+                }
+                state.advance_round(TimeToLive::from(1));
+            }
+            assert_eq!(state.round, Round(2000));
+            assert_eq!(state.round_sequence, Sequence(53320));
+            assert_eq!(state.sequence, Sequence(53320));
+        }
+
+        #[test]
+        fn test_sequence_wrap3() {
+            let total_rounds = 2000;
+            let max_probe_per_round = 20;
+            let mut state = TracerState::new(TimeToLive::from(1), Sequence(33000));
+            let mut rng = rand::thread_rng();
+            for _ in 0..total_rounds {
+                for _ in 0..rng.gen_range(0..max_probe_per_round) {
+                    state.next_probe();
+                }
+                state.advance_round(TimeToLive::from(1));
+            }
+        }
     }
 }
 
